<?php

/**
* @file
* Provides functions for use by the sports_scores module.
*/

/**
 * Given a sport, returns an array of games and scores
 * @param $sport
 */
function sports_scores_fetch_scores($sport) {
  $selected_parser = sports_scores_get_sports($sport);
  $http_result = drupal_http_request($selected_parser['url']);
  $scores = array();
  if ($http_result->code == 200) {
    $data = $http_result->data;
    $scores = call_user_func($selected_parser['parser'], $data, $sport);
    $scores['sport'] = $sport;
  }
  else {
    watchdog('sports_scores', 'Fetch of %url resulted in HTTP code %code.', array('%url' => $url, '%code' => $http_result->code), WATCHDOG_WARNING);
  }
  return $scores;
}


function sports_scores_load_scores($sport) {
  $scores = array();
  $scores['sport'] = $sport;
  $sql = "SELECT gid, final, in_progress, gametime, gamedate, home_team, visiting_team, home_score, visiting_score ";
  $sql .= "from {sports_scores_games} WHERE sport = '%s'";
  $result = db_query($sql, $sport);
  while ($g = db_fetch_array($result)) {
    $scores['games'][$g['gid']] = $g;
  }
  return $scores;
}


function sports_scores_process_game_updates($old, $new) {
  if ($old['sport'] != $new['sport']) {
    // Shouldn't ever be here
    return;
  }
  foreach (array_keys($new['games']) as $gid) {
    $oldgame = $old['games'][$gid];
    $newgame = $new['games'][$gid];

    if (!isset($old['games'][$gid])) {
      // Game doesn't exist.
      // Save it
      $g = (object)$newgame;
      $g->sport = $new['sport'];
      drupal_write_record('sports_scores_games', $g);

      if ( $newgame['final'] ) {
        // If game is already final, call trigger.
        trigger_sports_scores($newgame, 'game_score_final', $new['sport']);
      }
      elseif ($newgame['in_progress']) {
        // If game is already in progress, call trigger.
        trigger_sports_scores($newgame, 'game_score_update', $new['sport']);
      }
    }
    elseif ($oldgame['final']) {
      // Never do anything for a game that was already final.

    }
    elseif ($newgame['final'] && (! $oldgame['final'])) {
      // Game went final.
      // Save it
      $g = (object)$newgame;
      $g->sport = $new['sport'];
      drupal_write_record('sports_scores_games', $g, 'gid');
      // Call trigger.
      trigger_sports_scores($newgame, 'game_score_final', $new['sport']);
    }
    elseif ($newgame['home_score'] != $oldgame['home_score']
      || $newgame['visiting_score'] != $oldgame['visiting_score']) {
      // Scoring update
      // Save it
      $g = (object)$newgame;
      $g->sport = $new['sport'];
      drupal_write_record('sports_scores_games', $g, 'gid');
      // Call trigger.
      trigger_sports_scores($newgame, 'game_score_update', $new['sport']);
    }
  }
}


/* http://www.nfl.com/liveupdate/scorestrip/scorestrip.json
 * [["Sun","1:00","Final",,"TB","21","ATL","27",,,"54840",,"REG9","2010"], ...]
 * array(
 *   0 => Day of week,
 *   1 => Time
 *   2 => Final, Pregame OR final overtime
 *   3 => UNKNOWN
 *   4 => Visiting Team Identifier
 *   5 => Visiting Team Score
 *   6 => Home Team Identifier
 *   7 => Home Team Score
 *   8 => UNKNOWN
 *   9 => UNKNOWN
 *  10 => Game Identifier
 *  11 => UNKNOWN
 *  12 => Week Identifier
 *  13 => Year
 *  Missing: Clock, Ball, Redzone, and game action.

 * Another possible format
 * http://www.nfl.com/liveupdate/scorestrip/ss.xml
 */
function sports_scores_parse_nfl_dot_com_result($json, $sport) {
  $rawscores = array();
  $scores = array();
  // NFL.com has invalid json, this fixes it:
  while (strpos($json, ',,') > 0) {
    $json = str_replace(',,', ',null,', $json);
  }
  $rawscores = json_decode($json)->ss;
  if (module_exists('devel')) {
    $item = menu_get_item();
    if ($item['page_callback'] == 'sports_scores_dpm') {
      dpm($rawscores);
    }
  }

  foreach ($rawscores as $gameid => $rawgame) {
    $game = array();
    if (strpos(strtolower($rawgame[2]), 'final') !== FALSE) {
      $game['final'] = 1;
      $game['in_progress'] = 0;
      $game['home_score'] = $rawgame[7];
      $game['visiting_score'] = $rawgame[5];
    }
    elseif (strpos(strtolower($rawgame[2]), 'pregame') !== FALSE) {
      $game['final'] = 0;
      $game['in_progress'] = 0;
    }
    else {
      $game['final'] = 0;
      $game['in_progress'] = 1;
    }
    $game['home_team'] = $rawgame[6];
    $game['visiting_team'] = $rawgame[4];


    $dow = array('Mon' => 'Monday', 'Tue' => 'Tuesday', 'Wed' => 'Wednesday', 'Thu' => 'Thursday', 'Fri' => 'Friday', 'Sat' => 'Saturday', 'Sun' => 'Sunday');
    /* When the day has passed strtotime will flip to next week's date, so backup appropriately.
     * PHP doesn't have a GOOD way of deal with dynamic timezones yet.  Perhaps when we can assume PHP5.2+...
     * This also contains a 5 hour offset from EST => GMT, so hopefully
     * no games are within 5 hours of midnight or they will be on the wrong date.
     */
    // We know nfl time is EST or EDT, but not which one without guessing DST start/end dates.
    $game_start_unixsecs = strtotime($dow[$rawgame[0]] .' 1pm EST');
    // If the game is final and game start time is in the future, then we've already
    // passed the correct start day.  Backup 7 days.
    if ( $game_start_unixsecs > time() && $game['final'] == 1) {
      $game_start_unixsecs = $game_start_unixsecs - (7 * (60*60*24));
    }
    $game['gamedate'] = gmdate('Y-m-d', $game_start_unixsecs);


    // Two games in one day for the same teams at the same venue would be treated as one game.
    $game['gid'] = md5($sport .':' . $game['home_team'] . '@' . $game['visiting_team'] . ' ' . $game['gamedate']);

    $game['gametime'] = $rawgame[0] .' '. $rawgame[1];

    $scores['last_fetch'] = time();
    $scores['games'][$game['gid']] = $game;
  }

  return $scores;
}

function sports_scores_parse_espn_dot_com_result($query_string, $sport) {
  $rawscores = array();
  $scores = array();
  parse_str($query_string, $rawscores);
  if (module_exists('devel')) {
    $item = menu_get_item();
    if ($item['page_callback'] == 'sports_scores_dpm') {
      dpm($rawscores);
    }
  }
  foreach ($rawscores as $k => $v) {
    $matches = array();
    if (strpos($k, '_s_stamp')) {
      $scores['timestamp'] = $v;
      $scores['last_fetch'] = time();
    }
    elseif (preg_match('/_s_left(\d+)/', $k, $matches) ) {
      // It's a game.  It's either FINAL, in progress, or scheduled
      $game = array();
      // Remove ranking from team name ie '(4) Nebraska'
      $v = preg_replace('/\(\d+\) /', '', $v);
      if (preg_match('/^(.*) at (.*) \((.*)\)$/', $v, $matches)) {
        // It's scheduled
        $game += array(
          'final' => 0,
          'in_progress' => 0,
          'gametime' => $matches[3],
          'home_team' => $matches[2],
          'visiting_team' => $matches[1],
        );
      }
      elseif (preg_match('/^(.*) (\d+)   (.*) (\d+) \((FINAL.*)\)$/', $v, $matches)) {
        // It's final
        $game += array(
          'final' => 1,
          'in_progress' => 0,
          'gametime' => 'FINAL',
          'visiting_team' => str_replace('^', '', $matches[1]),
          'home_team' => str_replace('^', '', $matches[3]),
          'home_score' => $matches[2],
          'visiting_score' => $matches[4],
        );
      }
      elseif (preg_match('/^(.*) (\d+)   (.*) (\d+) \((.*)\)$/', $v, $matches)) {
        // It's final
        $game += array(
          'final' => 0,
          'in_progress' => 1,
          'gametime' => $matches[5],
          'home_team' => $matches[3],
          'visiting_team' => $matches[1],
          'home_score' => $matches[2],
          'visiting_score' => $matches[4],
        );
      }
      else {
        $game = array();
        watchdog('sports_scores', 'Parse of \'%string\' failed.', array('%string' => $v), WATCHDOG_WARNING);
      }
      $game['gamedate'] = gmdate('Y-m-d', strtotime('today 1pm EST'));

      // Two games in one day for the same teams at the same venue would be treated as one game.
      $game['gid'] = md5($sport .':' . $game['home_team'] . '@' . $game['visiting_team'] . ' ' . $game['gamedate']);
      $scores['games'][$game['gid']] = $game;
    }
  }
  return $scores;
}
